@@ -16,46 +16,50 @@ import shutil
 import sys
 import time
 import datetime
+import pkg_resources
 from functools import partial
 from multiprocessing import Pool
 from multiprocessing import cpu_count
-import pkg_resources
-from phylopypruner import fasta
-from phylopypruner import newick
-from phylopypruner import filtering
-from phylopypruner import decontamination
-from phylopypruner import mask_monophylies
-from phylopypruner import root
-from phylopypruner import report
-from phylopypruner import taxonomic_groups
-from phylopypruner.prune_paralogs import prune_paralogs
-from phylopypruner.summary import Summary
-from phylopypruner.summary import mk_sum_out_title
-from phylopypruner.log import Log
-from phylopypruner.settings import Settings
-from phylopypruner.supermatrix import Supermatrix
+from pathlib import Path
+from . import fasta
+from . import newick
+from . import filtering
+from . import decontamination
+from . import log
+from . import mask_monophylies
+from . import root
+from . import report
+from . import taxonomic_groups
+from . import settings
+from . import supermatrix
+from . import run
+from .prune_paralogs import prune_paralogs
+from .summary import Summary
+from .summary import mk_sum_out_title
 
-VERSION = pkg_resources.require("phylopypruner")[0].version
 FASTA_EXTENSIONS = {".fa", ".fas", ".fasta", ".fna", ".faa", ".fsa", ".ffn",
                     ".frn"}
 NW_EXTENSIONS = {".newick", ".nw", ".tre", ".tree", ".out", ".treefile"}
 ORTHO_STATS_PATH = "/output_alignment_stats.csv"
 HOMOLOG_STATS_PATH = "/input_alignment_stats.csv"
-ORTHOLOG_STATS_HEADER = "filename;otus;sequences;meanSeqLen;shortestSeq;longestSeq;\
-pctMissingData;alignmentLen\n"
-HOMOLOG_STATS_HEADER = "filename;otus;sequences;meanSeqLen;shortestSeq;longestSeq;\
-pctMissingData;alignmentLen;shortSequencesRemoved;longBranchesRemoved;\
-monophyliesMasked;nodesCollapsed;divergentOtusRemoved\n"
+ORTHOLOG_STATS_HEADER = "filename,otus,sequences,meanSeqLen,shortestSeq,longestSeq,\
+pctMissingData,alignmentLen\n"
+HOMOLOG_STATS_HEADER = "filename,otus,sequences,meanSeqLen,shortestSeq,longestSeq,\
+pctMissingData,alignmentLen,shortSequencesRemoved,longBranchesRemoved,\
+monophyliesMasked,nodesCollapsed,divergentOtusRemoved\n"
 ORTHO_STATS_PATH = "/output_alignment_stats.csv"
 LOG_PATH = "/phylopypruner.log"
 ORTHOLOGS_PATH = "/output_alignments"
 TIMESTAMP = datetime.datetime.now().strftime("%A, %d. %B %Y %I:%M%p")
-ABOUT = report.underline("PhyloPyPruner version {}".format(VERSION))
-ABOUT_LOG = "PhyloPyPruner version {}\n{}\n{}".format(
-    VERSION, TIMESTAMP, "-" * len(TIMESTAMP))
 NO_FILES = """You did not specify any input data. Use the flag '--dir', followed
 by the path to a directory, to point to a directory which contain your
 multiple sequence alignments (MSAs) and input trees."""
+# with open("VERSION") as version_file:
+#     version = version_file.read().strip()
+version = "1.1.2"
+ABOUT = report.underline("PhyloPyPruner version {}".format(version))
+ABOUT_LOG = "PhyloPyPruner version {}\n{}\n{}".format(
+    version, TIMESTAMP, "-" * len(TIMESTAMP))
 
 def _validate_input(msa, tree, tree_path):
     "Test to see if MSA and tree entries matches."
@@ -65,12 +69,15 @@ def _validate_input(msa, tree, tree_path):
     if set(descriptions).intersection(names) < set(descriptions):
         print("example tree names:", names[:2], file=sys.stderr)
         print("example sequences:", descriptions[:2], file=sys.stderr)
-        report.error("MSA names don't match tree \n   {}\n   {}".format(msa.filename, tree_path))
+        report.error("MSA names don't match tree \n   {}\n   {}".format(
+            msa.filename, tree_path))
+
 
 def _no_files(args):
     "Returns True if the required files are not provided."
     return not args.dir
 
+
 def _validate_arguments(args):
     "Performs a series of checks to validate the input before execution."
 
@@ -81,7 +88,7 @@ def _validate_arguments(args):
         errors = True
 
     if not args.outgroup and args.prune == "MO" or \
-        not args.outgroup and args.prune == "RT":
+       not args.outgroup and args.prune == "RT":
         report.error("pruning method is set to {}, but no outgroup has been \
 specified".format(args.prune))
         errors = True
@@ -114,13 +121,12 @@ either in percentage (1-100) or in decimal format between 0.0 - 1.0")
             args.min_gene_occupancy = args.min_gene_occupancy / 100
 
     if args.trim_divergent:
-        if args.trim_divergent < 0 or args.trim_divergent > 100:
-            report.error("the divergence threshold ('--trim-divergent') has to be \
-either in percentage (1-100) or in decimal format between 0.0 - 1.0")
-            errors = True
-        elif args.trim_divergent > 1:
-            # convert from percentage to floating point
+        if isinstance(args.trim_divergent, int):
             args.trim_divergent = args.trim_divergent / 100
+        if args.trim_divergent < 0:
+            report.error("the divergence threshold ('--trim-divergent') has to be \
+a positive number")
+            errors = True
 
     if args.min_len and args.min_len < 1:
         report.error("minimum sequence length ('--min-len') has to be a positive \
@@ -145,6 +151,7 @@ has to be a positive number")
     if errors:
         sys.exit()
 
+
 def _run(settings, msa, tree):
     """
     Takes a dictionary that specifies a set of settings as an input. The
@@ -155,39 +162,43 @@ def _run(settings, msa, tree):
     # test to see if the entries in the MSA and tree matches
     _validate_input(msa, tree, settings.nw_file)
 
-    log = Log(VERSION, msa, tree, settings)
+    run_log = log.Log(version, msa, tree, settings)
 
     # exclude taxa within the list settings.exclude
     if settings.exclude:
         tree = filtering.exclude(tree, settings.exclude)
 
     if filtering.too_few_otus(tree, settings.min_taxa):
-        return log
+        return run_log
 
     # remove short sequences
     if settings.min_len:
-        log.trimmed_seqs = filtering.trim_short_seqs(msa, tree, settings.min_len)
+        run_log.trimmed_seqs = filtering.trim_short_seqs(
+            msa, tree, settings.min_len)
 
     if filtering.too_few_otus(tree, settings.min_taxa):
-        return log
+        return run_log
 
     # trim long branches
     if settings.trim_lb:
-        log.lbs_removed = list(filtering.prune_long_branches(tree, settings.trim_lb))
+        run_log.lbs_removed = list(filtering.prune_long_branches(
+            tree, settings.trim_lb))
 
     if filtering.too_few_otus(tree, settings.min_taxa):
-        return log
+        return run_log
 
     # trim zero length branches
     if settings.trim_zero_len:
-        log.ultra_short_branches = filtering.trim_zero_len_branches(tree, settings.trim_zero_len)
+        run_log.ultra_short_branches = filtering.trim_zero_len_branches(
+            tree, settings.trim_zero_len)
 
     if filtering.too_few_otus(tree, settings.min_taxa):
-        return log
+        return run_log
 
     # collapse weakly supported nodes into polytomies
     if settings.min_support:
-        log.collapsed_nodes = filtering.collapse_nodes(tree, settings.min_support)
+        run_log.collapsed_nodes = filtering.collapse_nodes(
+            tree, settings.min_support)
 
     # mask monophyletic groups
     if settings.mask:
@@ -195,18 +206,18 @@ def _run(settings, msa, tree):
             tree, masked_seqs = mask_monophylies.pairwise_distance(tree)
         elif settings.mask == "longest":
             tree, masked_seqs = mask_monophylies.longest_isoform(msa, tree)
-        log.monophylies_masked = masked_seqs
+        run_log.monophylies_masked = masked_seqs
 
     # trim divergent sequences
     if settings.trim_divergent:
-        log.divergent, log.divergent_removed = decontamination.trim_divergent(
+        run_log.divergent, run_log.divergent_removed = decontamination.trim_divergent(
             tree, settings.trim_divergent, settings.include)
 
     if filtering.too_few_otus(tree, settings.min_taxa):
-        return log
+        return run_log
 
     # root by outgroup
-    rooted = False # True if outgroup rooting is successful
+    rooted = False   # True if outgroup rooting is successful
     if settings.outgroup:
         if not settings.prune == "MO":
             tree, rooted = root.outgroup(tree, settings.outgroup)
@@ -222,36 +233,23 @@ def _run(settings, msa, tree):
             tree, masked_seqs = mask_monophylies.pairwise_distance(tree)
         elif settings.mask == "longest":
             tree, masked_seqs = mask_monophylies.longest_isoform(msa, tree)
-        log.monophylies_masked.update(masked_seqs)
+        run_log.monophylies_masked.update(masked_seqs)
 
-    log.masked_tree = tree
+    run_log.masked_tree = tree
 
     # get a list of paralogs
-    log.paralogs = tree.paralogs()
+    run_log.paralogs = tree.paralogs()
 
     # prune paralogs
-    log.orthologs = prune_paralogs(settings.prune, tree,
+    run_log.orthologs = prune_paralogs(settings.prune, tree,
                                    settings.min_taxa, settings.outgroup)
 
     if settings.force_inclusion:
-        log.orthologs = filtering.force_inclusion(
-            log.orthologs, settings.force_inclusion)
+        run_log.orthologs = filtering.force_inclusion(
+            run_log.orthologs, settings.force_inclusion)
 
-    return log
+    return run_log
 
-def _get_orthologs(settings, directory="", dir_out=None):
-    fasta_path = "{}{}".format(directory, settings.fasta_file)
-    nw_path = "{}{}".format(directory, settings.nw_file)
-    msa = fasta.read(fasta_path)
-    nw_file = newick.read(nw_path)
-    log = _run(settings, msa, nw_file)
-    log.get_msas_out(dir_out)
-    log.report(dir_out)
-    return log
-
-def _run_for_file_pairs(corr_files, settings, dir_in, dir_out):
-    settings.fasta_file, settings.nw_file = corr_files
-    return _get_orthologs(settings, dir_in, dir_out)
 
 def file_pairs_from_directory(directory):
     """Takes the path to a directory as an input. Finds corresponding files, or
@@ -324,12 +322,13 @@ def file_pairs_from_directory(directory):
 
     return [file_pairs[file_pair] for file_pair in file_pairs]
 
+
 def parse_args():
     "Parse the arguments provided by the user."
     parser = argparse.ArgumentParser(description=__doc__)
     parser.add_argument("-v", "-V", "--version",
                         action="version",
-                        version=str(VERSION),
+                        version=str(version),
                         help="display the version number and exit")
     parser.add_argument("--overwrite",
                         default=False,
@@ -339,154 +338,160 @@ def parse_args():
     parser.add_argument("--no-plot",
                         default=False,
                         action="store_true",
-                        help="do not generate plots (faster)")
+                        help="do not generate any plots (faster)")
     parser.add_argument("--wrap",
-                        metavar="COLUMN",
+                        metavar="<number>",
                         default=None,
                         type=int,
-                        help="wrap output sequences at this column instead \
+                        help="wrap output sequences at column <number>, instead\
                               of writing each sequence to a single line")
     parser.add_argument("--threads",
-                        metavar="COUNT",
+                        metavar="<number>",
                         default=None,
                         type=int,
-                        help="use this many threads instead of up to 10")
+                        help="use <number> threads instead of up to 10 threads")
     parser.add_argument("--output",
-                        metavar="DIRECTORY",
+                        metavar="<directory>",
                         type=str,
                         default=None,
-                        help="save output files to this directory instead of \
+                        help="save output files to <directory>, instead of\
                               the input directory")
 
     group = parser.add_argument_group("input data")
     group.add_argument("--dir",
-                       metavar="DIRECTORY",
+                       metavar="<directory>",
                        type=str,
                        default=None,
-                       help="a folder containing 1+ alignment and tree files")
+                       help="a <directory> containing 1+ alignment and tree files")
 
-    group = parser.add_argument_group("filters")
-    group.add_argument("--min-taxa",
-                       metavar="COUNT",
-                       type=int,
-                       default=4,
-                       help="discard output alignments with fewer OTUs than \
-                             this (4 by default)")
-    group.add_argument("--min-len",
-                       metavar="LENGTH",
-                       default=None,
-                       type=int,
-                       help="remove sequences with fewer bases than this")
-    group.add_argument("--min-support",
-                       metavar="SUPPORT",
-                       default=None,
-                       type=float,
-                       help="collapse nodes with less support than this into \
-                             polytomies")
-    group.add_argument("--trim-lb",
-                       default=None,
-                       metavar="FACTOR",
-                       type=float,
-                       help="remove branches longer than FACTOR standard \
-                             deviations of all branches")
-    group.add_argument("--min-pdist",
-                       default=None,
-                       metavar="DISTANCE",
-                       type=float,
-                       help="remove sequence pairs with less tip-to-tip \
-                             distance than this")
-    group.add_argument("--include",
-                       nargs="+",
-                       metavar="OTU",
-                       default=None,
-                       type=str,
-                       help="include these OTUs, even if deemed problematic \
-                             by '--trim-freq-paralogs' or '--trim-divergent'")
+    group = parser.add_argument_group("prefilters")
     group.add_argument("--exclude",
                        nargs='+',
                        metavar="OTU",
                        default=None,
                        type=str,
                        help="exclude these OTUs")
-    group.add_argument("--force-inclusion",
+    group.add_argument("--min-len",
+                       metavar="<number>",
+                       default=None,
+                       type=int,
+                       help="remove sequences which are shorter than <number> bases")
+    group.add_argument("--trim-lb",
+                       default=None,
+                       metavar="<factor>",
+                       type=float,
+                       help="remove branches longer than <factor> standard\
+                             deviations of all branches")
+    group.add_argument("--min-pdist",
+                       default=None,
+                       metavar="<distance>",
+                       type=float,
+                       help="remove sequence pairs with less tip-to-tip\
+                             distance than <distance>")
+    group.add_argument("--min-support",
+                       metavar="<percentage>",
+                       default=None,
+                       type=float,
+                       help="collapse nodes with less support than\
+                             <percentage> into polytomies")
+    group.add_argument("--mask",
+                       default="pdist",
+                       type=str,
+                       choices=["longest", "pdist"],
+                       help="if 2+ sequences from a single OTU forms a clade,\
+                             choose which sequence to keep using this method")
+    group.add_argument("--trim-divergent",
+                       default=None,
+                       metavar="<percentage>",
+                       type=float,
+                       help="for each alignment: discard all sequences from an OTU on a\
+                       per-alignment-basis, if the ratio between the largest\
+                       pairwise distance of sequences from this OTU and\
+                       the average pairwise distance of sequences from this\
+                       OTU to other's exceed this <percentage>")
+    group.add_argument("--trim-freq-paralogs",
+                       default=None,
+                       metavar="<factor>",
+                       type=float,
+                       help="exclude OTUs with more paralogy frequency (PF)\
+                             than <factor> standard deviations of all PFs")
+    group.add_argument("--include",
                        nargs="+",
                        metavar="OTU",
                        default=None,
                        type=str,
-                       help="discard output alignments where these OTUs are \
-                             missing")
-    group.add_argument("--min-otu-occupancy",
-                       metavar="OCCUPANCY",
-                       default=None,
-                       type=float,
-                       help="do not include OTUs with less occupancy than \
-                       this")
-    group.add_argument("--min-gene-occupancy",
-                       metavar="OCCUPANCY",
-                       default=None,
-                       type=float,
-                       help="discard output alignments with less occupancy \
-                       than this")
+                       help="include these OTUs, even if deemed problematic\
+                             by '--trim-freq-paralogs' or '--trim-divergent'")
 
-    group = parser.add_argument_group("tree-based orthology inference")
+    group = parser.add_argument_group("rooting")
     group.add_argument("--outgroup",
                        nargs="+",
                        metavar="OTU",
                        default=None,
                        type=str,
-                       help="root trees using these OTUs if at least one OTU \
-                             is present and if all present OTUs are \
+                       help="root trees using these OTUs if at least one OTU\
+                             is present and if all present OTUs are\
                              non-repetetive and form a clade")
     group.add_argument("--root",
                        default=None,
                        type=str,
                        choices=["midpoint"],
-                       help="root trees using this method in cases where no \
-                             outgroup rooting is not performed")
-    group.add_argument("--mask",
-                       default="pdist",
-                       type=str,
-                       choices=["longest", "pdist"],
-                       help="if 2+ sequences from a single OTU forms a clade, \
-                             choose which sequence to keep using this method")
+                       help="root trees using this method when outgroup\
+                             rooting was not performed")
+
+    group = parser.add_argument_group("paralogy pruning")
     group.add_argument("--prune",
                        default="LS",
                        type=str,
-                       choices=["LS", "MI", "MO", "RT", "1to1", "TD"],
-                       help="set the paralogy pruning method (default: LS)")
+                       choices=["LS", "MI", "MO", "RT", "1to1", "OTO"],
+                       help="select the paralogy pruning method (default: LS)")
 
-    group = parser.add_argument_group("decontamination")
-    group.add_argument("--trim-freq-paralogs",
+    group = parser.add_argument_group("postfilters")
+    group.add_argument("--force-inclusion",
+                       nargs="+",
+                       metavar="OTU",
                        default=None,
-                       metavar="FACTOR",
-                       type=float,
-                       help="exclude OTUs with more paralogy frequency (PF) \
-                             than FACTOR standard deviations of all PFs")
-    group.add_argument("--trim-divergent",
+                       type=str,
+                       help="discard output alignments where these OTUs are\
+                             missing")
+    group.add_argument("--min-taxa",
+                       metavar="<number>",
+                       type=int,
+                       default=4,
+                       help="discard output alignments with fewer OTUs than\
+                             <number> (4 by default)")
+    group.add_argument("--min-otu-occupancy",
+                       metavar="<percentage>",
                        default=None,
-                       metavar="PERCENTAGE",
                        type=float,
-                       help="for each alignment: discard all sequences from an OTU on a \
-                       per-alignment-basis, if the ratio between the largest \
-                       pairwise distance of sequences from this OTU and \
-                       the average pairwise distance of sequences from this \
-                       OTU to other's exceed this percentage")
+                       help="do not include OTUs with less occupancy than\
+                             <percentage>")
+    group.add_argument("--min-gene-occupancy",
+                       metavar="<percentage>",
+                       default=None,
+                       type=float,
+                       help="discard output alignments with less occupancy\
+                             than <percentage>")
+
+    group = parser.add_argument_group("post-processing")
     group.add_argument("--subclades",
                        default=None,
                        metavar="FILE",
                        type=str,
-                       help="specify a set of subclades within this file \
+                       help="specify a set of subclades within this file\
                              and analyse their overall stability")
     group.add_argument("--jackknife",
                        default=False,
                        action="store_true",
-                       help="exclude each OTU one by one, rerun the whole \
-                             analysis and generate statistics for each \
+                       help="exclude each OTU one by one, rerun the whole\
+                             analysis and generate statistics for each\
                              subsample")
     return parser.parse_args(args=None if sys.argv[1:] else ['--help'])
 
+
 def main():
     "Parse args, run filter and infer orthologs."
+    START_TIME = time.time()
     args = parse_args()
     print(ABOUT, file=sys.stderr)
     _validate_arguments(args)
@@ -497,7 +502,7 @@ def main():
         groups = taxonomic_groups.read(args.subclades)
         args.subclades = groups
 
-    settings = Settings(args)
+    parameters = settings.Settings(args)
 
     if args.threads:
         threads = args.threads
@@ -506,7 +511,7 @@ def main():
 
     dir_out = None
     if args.output:
-        dir_out = args.output.rstrip("/") # get rid of trailing slash in path
+        dir_out = args.output.rstrip("/")  # get rid of trailing slash in path
     elif args.dir:
         dir_out = str(args.dir).rstrip("/")
     dir_out = dir_out + "/phylopypruner_output"
@@ -533,8 +538,6 @@ run, overwrite these files?", display=False)):
     with open(dir_out + HOMOLOG_STATS_PATH, "w") as homo_stats_file:
         homo_stats_file.write(HOMOLOG_STATS_HEADER)
 
-    parameters_used = settings.report(args.dir, dir_out + LOG_PATH)
-
     if not args.dir[-1] == "/":
         dir_in = args.dir + "/"
     else:
@@ -546,16 +549,16 @@ run, overwrite these files?", display=False)):
         sys.exit()
 
     # count and report the number of threads used
-    threads_available = cpu_count()
+    thread_count = cpu_count()
     if args.threads:
-        threads = args.threads if threads <= threads_available else threads_available
+        threads = args.threads if threads <= thread_count else thread_count
     else:
-        threads = threads_available if threads_available <= 10 else 10
+        threads = thread_count if thread_count <= 10 else 10
     pool = Pool(processes=threads)
-    part_run = partial(_run_for_file_pairs, settings=settings,
+    part_run = partial(run.run_for_file_pairs, settings=parameters,
                        dir_in=dir_in, dir_out=dir_out)
     report.progress_bar("using {} out of {} available threads".format(
-        threads, threads_available))
+        threads, thread_count))
     print("", file=sys.stderr)
 
     file_pairs = file_pairs_from_directory(dir_in)
@@ -596,7 +599,7 @@ more relaxed settings")
     # Generate a list of OTUs to exclude and exclude them from the summary.
     if otus_to_exclude:
         if args.include:
-            otus_to_exclude = [otu for otu in otus_to_exclude if not otu in
+            otus_to_exclude = [otu for otu in otus_to_exclude if otu not in
                                args.include]
         summary = decontamination.prune_by_exclusion(
             summary, otus_to_exclude, dir_out, threads)
@@ -605,20 +608,20 @@ more relaxed settings")
     otus_to_exclude, genes_to_exclude = summary.matrix_occupancy(
         dir_out, args.min_otu_occupancy, args.min_gene_occupancy, args.no_plot)
 
-    if otus_to_exclude:
-        summary = decontamination.exclude_otus(summary, otus_to_exclude)
-        summary.discarded_otus = otus_to_exclude
-
     if genes_to_exclude:
         summary = decontamination.exclude_genes(summary, genes_to_exclude)
         summary.discarded_genes = genes_to_exclude
 
+    if otus_to_exclude:
+        summary = decontamination.exclude_otus(summary, otus_to_exclude)
+        summary.discarded_otus = otus_to_exclude
+
     # Remove gap-only columns from the output alignments.
     summary = summary.remove_gap_only_columns()
 
-    if settings.taxonomic_groups:
+    if parameters.taxonomic_groups:
         decontamination.score_monophyly(
-            summary, settings.taxonomic_groups, dir_out)
+            summary, parameters.taxonomic_groups, dir_out)
 
     # Perform taxon jackknifing.
     if args.jackknife:
@@ -628,8 +631,8 @@ more relaxed settings")
     summary.write_msas(args.wrap)
 
     # concatenate output alignments into a supermatrix
-    supermatrix = Supermatrix(dir_out)
-    supermatrix.partitions_from_summary(summary, dir_out)
+    matrix = supermatrix.Supermatrix(dir_out)
+    matrix.partitions_from_summary(summary, dir_out)
 
     # print the output
     path_out = report.print_path(dir_out, display=False)
@@ -637,11 +640,12 @@ more relaxed settings")
 
     # print settings
     print("", file=sys.stderr)
-    settings.print_settings()
+    parameters.print_settings()
 
     # print alignment statistics
     print(ortholog_report)
-    run_time = "\ncompleted in {} seconds".format(round(time.time() - START_TIME, 2))
+    run_time = "\ncompleted in {} seconds".format(
+        round(time.time() - START_TIME, 2))
 
     with open(dir_out + LOG_PATH, "a") as log_file:
         ortholog_report = ortholog_report.replace("\33[0m", "")
@@ -652,7 +656,13 @@ more relaxed settings")
             " ".join([arg for arg in sys.argv]))
         log_file.write("\n" + run_time)
 
+
+def entry():
+    "Entry point for command-line script"
+    main()
+    return 0
+
+
 if __name__ == "__main__":
     sys.path.insert(0, os.path.abspath('..'))
-    START_TIME = time.time()
     main()
