@@ -4,26 +4,24 @@ from __future__ import absolute_import
 import os
 import datetime
 from collections import defaultdict
-from phylopypruner.report import underline
-from phylopypruner import fasta
-from phylopypruner import report
-from phylopypruner import plot
+from . import fasta
+from . import report
+from . import plot
+from .report import underline
 try:
     import matplotlib as mpl
-    if not "DISPLAY" in os.environ:
+    if "DISPLAY" not in os.environ:
         mpl.use("agg")
-    import matplotlib.pyplot as plt
     MATPLOTLIB = True
 except ImportError:
-    report.tip("install Matplotlib (https://matplotlib.org/) to generate \
-plots")
     MATPLOTLIB = False
 TIMESTAMP = datetime.datetime.now().strftime("%Y-%m-%d")
-SUM_HEADER = "id;alignments;sequences;otus;meanSequences;meanOtus;meanSeqLen;\
-shortestSeq;longestSeq;pctMissingData;catAlignmentLen\n"
+SUM_HEADER = "id,alignments,sequences,otus,meanSequences,meanOtus,meanSeqLen,\
+shortestSeq,longestSeq,pctMissingData,catAlignmentLen\n"
 SUM_PATH = "/supermatrix_stats.csv"
 OCCUPANCY_PLOT_FILE = "/plots/occupancy_matrix.png"
 FREQ_CSV_FILE = "/otu_stats.csv"
+SPECIES_LIST_FILE = "/otu_list.txt"
 GAP_CHARACTERS = {"-", "?", "x"}
 RESOLUTION = 72
 XFONT_SIZE = 2.5
@@ -76,7 +74,6 @@ class Summary(object):
     def discarded_genes(self, value):
         self._discarded_genes = value
 
-
     def remove_gap_only_columns(self):
         """Iterate over the output alignments within this summary and remove
         positions where no residues are present.
@@ -96,14 +93,20 @@ class Summary(object):
                 # Generate a list of booleans for each alignment: True = one
                 # or more residues present at column, False = residues absent.
                 presence = [False] * len(msa.sequences[0])
+                residues = len(presence)
 
                 # Identify columns where no residues are present.
                 for sequence in msa.sequences:
                     for index, position in enumerate(sequence.sequence_data):
-                        if not position in GAP_CHARACTERS:
+                        if index > residues:
+                            print(report.error("Uneven length in alignment \
+                                {}".format(msa.filename)))
+                            break
+
+                        if position not in GAP_CHARACTERS:
                             presence[index] = True
 
-                if not False in presence:
+                if False not in presence:
                     continue
 
                 for sequence in msa.sequences:
@@ -294,11 +297,17 @@ class Summary(object):
             threshold = _std(list(paralog_freq.values())) * factor
 
         with open(dir_out + FREQ_CSV_FILE, "w") as csv_out:
-            csv_out.write("otu;paralogyFrequency;timesAboveDivergenceThreshold\n")
+            csv_out.write("otu,paralogyFrequency,timesAboveDivergenceThreshold\n")
             for otu, freq in paralog_freq.items():
-                csv_out.write("{};{};{}\n".format(
+                csv_out.write("{},{},{}\n".format(
                     otu, freq, divergent[otu]))
 
+        # In addition to the OTU-stats, create a list of all the OTUs, which
+        # can be useful for automating subsequent steps.
+        with open(dir_out + SPECIES_LIST_FILE, "w") as species_list:
+            for otu, _ in paralog_freq.items():
+                species_list.write("{}\n".format(otu))
+
         otus = list(paralog_freq.keys())
         indices = range(len(otus))
         freq = list(paralog_freq.values())
@@ -345,7 +354,7 @@ class Summary(object):
         report : str
             A report of the statistics within this file.
         row : str
-            A CSV-row where each column is separated by a semicolon, ';'.
+            A CSV-row where each column is separated by a semicolon, ','.
         """
         no_of_alignments = 0
         no_of_seqs = 0
@@ -401,7 +410,7 @@ class Summary(object):
         if not longest:
             longest = 0
 
-        row = "{};{};{};{};{};{};{};{};{};{};{}\n".format(
+        row = "{},{},{},{},{},{},{},{},{},{},{}\n".format(
             title,
             no_of_alignments,
             no_of_seqs,
@@ -439,7 +448,7 @@ class Summary(object):
         report : str
             A report of the statistics within this file.
         row : str
-            A CSV-row where each column is separated by a semicolon, ';'.
+            A CSV-row where each column is separated by a semicolon, ','.
         """
 
         pct_missing = 0.0
@@ -506,17 +515,26 @@ class Summary(object):
             stats["avg_seq_len"] = int(stats["seq_lens"] / stats["sequences"])
 
         # determine the column width from the longest statistic
-        col_width = max(
-            len(str(stats["sequences"])),
-            len(str(stats["alignments"])),
-            len(str(stats["cat_alignment_len"])),
-            len(str(stats["no_of_otus"])),
-            len(str(stats["longest"])),
-            len(str(homolog_stats[1])),
-            len(str(homolog_stats[2])),
-            len(str(homolog_stats[8])),
-            len(str(homolog_stats[10]))
-            ) + 1
+        if homolog_stats:
+            col_width = max(
+                len(str(stats["sequences"])),
+                len(str(stats["alignments"])),
+                len(str(stats["cat_alignment_len"])),
+                len(str(stats["no_of_otus"])),
+                len(str(stats["longest"])),
+                len(str(homolog_stats[1])),
+                len(str(homolog_stats[2])),
+                len(str(homolog_stats[8])),
+                len(str(homolog_stats[10]))
+                ) + 1
+        else:
+            col_width = max(
+                len(str(stats["sequences"])),
+                len(str(stats["alignments"])),
+                len(str(stats["cat_alignment_len"])),
+                len(str(stats["no_of_otus"])),
+                len(str(stats["longest"])),
+                ) + 1
 
         if col_width < 4:
             col_width = 4
@@ -530,6 +548,24 @@ class Summary(object):
         methods_header = "Methods summary:\n  " +\
                 underline("{:29s} {:{}s}    {:{}s}".format(
                     name, "No. removed", col_width, "% of input", col_width))
+
+        row = "{},{},{},{},{},{},{},{},{},{},{}\n".format(
+            title,
+            stats["alignments"],
+            stats["sequences"],
+            stats["no_of_otus"],
+            stats["avg_no_of_seqs"],
+            stats["avg_no_of_seqs"],
+            stats["avg_seq_len"],
+            stats["shortest"],
+            stats["longest"],
+            stats["missing_data"],
+            stats["cat_alignment_len"])
+
+
+        if not homolog_stats:
+            return "", row
+
         stats_report = """
 {}
   No. of alignments                 {:{}d}   {:{}d}
@@ -574,19 +610,6 @@ class Summary(object):
       100 * stats["genes_removed_count"] / homolog_stats[2], col_width
   )
 
-        row = "{};{};{};{};{};{};{};{};{};{};{}\n".format(
-            title,
-            stats["alignments"],
-            stats["sequences"],
-            stats["no_of_otus"],
-            stats["avg_no_of_seqs"],
-            stats["avg_no_of_seqs"],
-            stats["avg_seq_len"],
-            stats["shortest"],
-            stats["longest"],
-            stats["missing_data"],
-            stats["cat_alignment_len"])
-
         return stats_report, row
 
     def report(self, title, dir_out, homolog_stats=None):
@@ -606,7 +629,10 @@ class Summary(object):
         report : str
             Overview statistics of the summary.
         """
-        report, row = self.alignment_stats("Description", title, homolog_stats)
+        if homolog_stats:
+            report, row = self.alignment_stats("Description", title, homolog_stats)
+        else:
+            report, row = self.alignment_stats("Description", title)
 
         with open(dir_out + SUM_PATH, "a") as sum_out_file:
             sum_out_file.write(row)
