@@ -1,5 +1,6 @@
 "Module for working with a phylogenetic tree."
 
+from __future__ import absolute_import
 import re
 from collections import deque
 
@@ -145,7 +146,7 @@ class TreeNode(object):
         if not isinstance(node, TreeNode):
             raise TypeError("{} must be a Treeroot object".format(node))
 
-        if not node in self.traverse_preorder():
+        if node not in self.traverse_preorder():
             raise AssertionError("{} is not present within tree \
                     {}".format(node, self))
 
@@ -284,7 +285,9 @@ class TreeNode(object):
         -------
         True if the number of children in this node == 1, else False.
         """
-        return not self.is_leaf() and not self.is_root() and len(self.children) < 2
+        return not self.is_leaf() and\
+            not self.is_root() and\
+            len(self.children) < 2
 
     def has_monofurcations(self):
         """Returns True if there are any non-root nodes which are
@@ -589,7 +592,7 @@ class TreeNode(object):
         :param attributes: A list of node attributes to shown in the
             ASCII representation.
         """
-        (lines, mid) = self._asciiArt(show_internal=show_internal,
+        (lines, _) = self._asciiArt(show_internal=show_internal,
                                       compact=compact, attributes=attributes)
         return '\n'+'\n'.join(lines)
 
